@@ -12,10 +12,13 @@ from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile as \
      ZopeViewPageTemplateFile
 
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
+from Products.PageTemplates.PageTemplateFile import PageTemplate
+from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile as \
      FiveViewPageTemplateFile
 
 from five.pt.pagetemplate import ViewPageTemplateFile
+from five.pt.pagetemplate import BaseTemplate
 from five.pt.pagetemplate import BaseTemplateFile
 
 from Acquisition import aq_base
@@ -23,6 +26,8 @@ from Acquisition import aq_parent
 from Acquisition.interfaces import IAcquirer
 from Acquisition import ImplicitAcquisitionWrapper
 
+from ComputedAttribute import ComputedAttribute
+
 try:
     from Products.Five.browser.pagetemplatefile import BoundPageTemplate
 except ImportError:
@@ -56,38 +61,61 @@ def get_bound_template(self, instance, type):
     if instance is None:
         return self
 
-    template = getattr(self, '_template', _marker)
+    template = getattr(self, '_v_template', _marker)
     if template is _marker:
-        self._template = template = ViewPageTemplateFile(self.filename)
+        self._v_template = template = ViewPageTemplateFile(self.filename)
 
     return BoundPageTemplate(template, instance)
 
 def call_template(self, *args, **kw):
-    template = getattr(self, '_template', _marker)
-    if template is _marker:
-        self._template = template = BaseTemplateFile(self.filename)
+    template = getattr(self, '_v_template', _marker)
+    if template is _marker or self._text != template.body:
+        self._v_template = template = BaseTemplate(self._text)
+
+    return template(self, *args, **kw)
+
+def call_template_and_wrap(self, *args, **kw):
+    template = getattr(self, '_v_template', _marker)
+    if template is _marker or self._text != template.body:
+        self._v_template = template = BaseTemplate(self._text)
 
     if IAcquirer.providedBy(template):
-        template = template.__of__(self)
+        template = template.__of__(aq_parent(self))
+    else:
+        template = ImplicitAcquisitionWrapper(template, aq_parent(self))
+
+    return template(self, *args, **kw)
+
+def call_template_file(self, *args, **kw):
+    template = getattr(self, '_v_template', _marker)
+    if template is _marker:
+        self._v_template = template = BaseTemplateFile(self.filename)
+
+    if IAcquirer.providedBy(template):
+        template = template.__of__(aq_parent(self))
     else:
         template = ImplicitAcquisitionWrapper(template, aq_parent(self))
 
     return template(self, *args, **kw)
 
 def get_macros(self):
-    template = getattr(self, '_template', _marker)
+    template = getattr(self, '_v_template', _marker)
     if template is _marker:
-        self._template = template = BaseTemplateFile(self.filename)
+        self._v_template = template = BaseTemplateFile(self.filename)
 
     if IAcquirer.providedBy(template):
-        return template.__of__(self).macros
+        return template.__of__(aq_parent(self)).macros
     else:
         return template.macros
 
 FiveViewPageTemplateFile.__get__ = get_bound_template
 ZopeViewPageTemplateFile.__get__ = get_bound_template
-PageTemplateFile.__call__ = call_template
+PageTemplate.__call__ = call_template
+PageTemplate.macros = ComputedAttribute(get_macros, 1)
+PageTemplateFile.__call__ = call_template_file
 PageTemplateFile.macros = property(get_macros)
+ZopePageTemplate.__call__ = call_template_and_wrap
+ZopePageTemplate.macros = ComputedAttribute(get_macros, 1)
 
 try:
     from five.grok.components import ZopeTwoPageTemplate
